﻿using AutoMapper;
using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess.SampleSchema;
using RemObjects.DataAbstract.Linq;

namespace DataAccess
{
	public sealed class DataLayer : IDataLayer
	{
		private readonly IMapper _mapper;
		private readonly LinqDataAdapter _adapter;
		private readonly DataService _service;

		public DataLayer(IMapper mapper)
		{
			this._mapper = mapper;

			this._service = new DataService();
			this._service.Activate(Guid.Empty, false);

			this._adapter = new LinqLocalDataAdapter(this._service);
		}

		public void Dispose()
		{
			if (this._service.IsActive)
			{
				this._service.Deactivate(Guid.Empty);
			}

			GC.SuppressFinalize(this);
		}

		public void CreateCustomer(CustomerDTO customerDto)
		{
			var customer = this._mapper.Map<Customers>(customerDto);
			this._adapter.GetTable<Customers>().InsertRow(customer);
			this._adapter.ApplyChanges();
		}

		public void UpdateCustomer(CustomerDTO customerDto)
		{
			var customer = this._mapper.Map<Customers>(customerDto);

			this._adapter.GetTable<Customers>().UpdateRow(customer);
			this._adapter.ApplyChanges();
		}

		public void DeleteCustomer(string id)
		{
			this._adapter.GetTable<Customers>().DeleteRow(new Customers() { Id = id });
			this._adapter.ApplyChanges();
		}

		public IList<CustomerDTO> GetAllCustomers()
		{
			var query = this._adapter
				.GetTable<Customers>()
				.Select(c => c);
			
			var data = query.ToList<Customers>();

			return this._mapper.Map<IList<CustomerDTO>>(data);
		}

		public CustomerDTO GetCustomer(string id)
		{
			var customer = this._adapter.GetTable<Customers>()
				.Select(c => c)
				.FirstOrDefault(c => c.Id == id);

			if (customer == null)
			{
				return null;
			}

			var customerDto = this._mapper.Map<CustomerDTO>(customer);

			return customerDto;
		}
	}
}